@@ -459,6 +459,7 @@ clutter_bin_layout_allocate (ClutterLayoutManager   *manager,
           continue;
         }
 
+      child_width = child_height = 0;
       request = clutter_actor_get_request_mode (child);
       if (request == CLUTTER_REQUEST_HEIGHT_FOR_WIDTH)
         {
@@ -558,9 +559,13 @@ clutter_bin_layout_set_container (ClutterLayoutManager *manager,
                                   ClutterContainer     *container)
 {
   ClutterBinLayoutPrivate *priv;
+  ClutterLayoutManagerClass *parent_class;
 
   priv = CLUTTER_BIN_LAYOUT (manager)->priv;
   priv->container = container;
+
+  parent_class = CLUTTER_LAYOUT_MANAGER_CLASS (clutter_bin_layout_parent_class);
+  parent_class->set_container (manager, container);
 }
 
 static void
