@@ -18,9 +18,7 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library. If not, see <http://www.gnu.org/licenses/>.
  */
 
 /**
@@ -42,7 +40,6 @@
 #include <math.h>
 
 #include <pango/pango-attributes.h>
-#include <gobject/gvaluecollector.h>
 
 #include "clutter-main.h"
 #include "clutter-color.h"
@@ -629,20 +626,34 @@ static void
 clutter_value_transform_color_string (const GValue *src,
                                       GValue       *dest)
 {
-  gchar *string = clutter_color_to_string (src->data[0].v_pointer);
+  const ClutterColor *color = g_value_get_boxed (src);
 
-  g_value_take_string (dest, string);
+  if (color)
+    {
+      gchar *string = clutter_color_to_string (color);
+
+      g_value_take_string (dest, string);
+    }
+  else
+    g_value_set_string (dest, NULL);
 }
 
 static void
 clutter_value_transform_string_color (const GValue *src,
                                       GValue       *dest)
 {
-  ClutterColor color = { 0, };
+  const char *str = g_value_get_string (src);
 
-  clutter_color_from_string (&color, g_value_get_string (src));
+  if (str)
+    {
+      ClutterColor color = { 0, };
 
-  clutter_value_set_color (dest, &color);
+      clutter_color_from_string (&color, str);
+
+      clutter_value_set_color (dest, &color);
+    }
+  else
+    clutter_value_set_color (dest, NULL);
 }
 
 GType
@@ -666,82 +677,6 @@ clutter_color_get_type (void)
   return _clutter_color_type;
 }
 
-static void
-clutter_value_init_color (GValue *value)
-{
-  value->data[0].v_pointer = NULL;
-}
-
-static void
-clutter_value_free_color (GValue *value)
-{
-  if (!(value->data[1].v_uint & G_VALUE_NOCOPY_CONTENTS))
-    clutter_color_free (value->data[0].v_pointer);
-}
-
-static void
-clutter_value_copy_color (const GValue *src,
-                          GValue       *dest)
-{
-  dest->data[0].v_pointer = clutter_color_copy (src->data[0].v_pointer);
-}
-
-static gpointer
-clutter_value_peek_color (const GValue *value)
-{
-  return value->data[0].v_pointer;
-}
-
-static gchar *
-clutter_value_collect_color (GValue      *value,
-                             guint        n_collect_values,
-                             GTypeCValue *collect_values,
-                             guint        collect_flags)
-{
-  if (!collect_values[0].v_pointer)
-      value->data[0].v_pointer = NULL;
-  else
-    {
-      if (collect_flags & G_VALUE_NOCOPY_CONTENTS)
-        {
-          value->data[0].v_pointer = collect_values[0].v_pointer;
-          value->data[1].v_uint = G_VALUE_NOCOPY_CONTENTS;
-        }
-      else
-        {
-          value->data[0].v_pointer =
-            clutter_color_copy (collect_values[0].v_pointer);
-        }
-    }
-
-  return NULL;
-}
-
-static gchar *
-clutter_value_lcopy_color (const GValue *value,
-                           guint         n_collect_values,
-                           GTypeCValue  *collect_values,
-                           guint         collect_flags)
-{
-  ClutterColor **color_p = collect_values[0].v_pointer;
-
-  if (!color_p)
-    return g_strdup_printf ("value location for '%s' passed as NULL",
-                            G_VALUE_TYPE_NAME (value));
-
-  if (!value->data[0].v_pointer)
-    *color_p = NULL;
-  else
-    {
-      if (collect_flags & G_VALUE_NOCOPY_CONTENTS)
-        *color_p = value->data[0].v_pointer;
-      else
-        *color_p = clutter_color_copy (value->data[0].v_pointer);
-    }
-
-  return NULL;
-}
-
 /**
  * clutter_value_set_color:
  * @value: a #GValue initialized to #CLUTTER_TYPE_COLOR
@@ -757,7 +692,7 @@ clutter_value_set_color (GValue             *value,
 {
   g_return_if_fail (CLUTTER_VALUE_HOLDS_COLOR (value));
 
-  value->data[0].v_pointer = clutter_color_copy (color);
+  g_value_set_boxed (value, color);
 }
 
 /**
@@ -775,7 +710,7 @@ clutter_value_get_color (const GValue *value)
 {
   g_return_val_if_fail (CLUTTER_VALUE_HOLDS_COLOR (value), NULL);
 
-  return value->data[0].v_pointer;
+  return g_value_get_boxed (value);
 }
 
 static void
@@ -796,10 +731,11 @@ param_color_finalize (GParamSpec *pspec)
 
 static void
 param_color_set_default (GParamSpec *pspec,
-                        GValue     *value)
+                         GValue     *value)
 {
-  value->data[0].v_pointer = CLUTTER_PARAM_SPEC_COLOR (pspec)->default_value;
-  value->data[1].v_uint = G_VALUE_NOCOPY_CONTENTS;
+  const ClutterColor *default_value =
+    CLUTTER_PARAM_SPEC_COLOR (pspec)->default_value;
+  clutter_value_set_color (value, default_value);
 }
 
 static gint
@@ -807,30 +743,24 @@ param_color_values_cmp (GParamSpec   *pspec,
                         const GValue *value1,
                         const GValue *value2)
 {
-  guint32 color1, color2;
+  const ClutterColor *color1 = g_value_get_boxed (value1);
+  const ClutterColor *color2 = g_value_get_boxed (value2);
+  int pixel1, pixel2;
 
-  color1 = clutter_color_to_pixel (value1->data[0].v_pointer);
-  color2 = clutter_color_to_pixel (value2->data[0].v_pointer);
+  if (color1 == NULL)
+    return color2 == NULL ? 0 : -1;
 
-  if (color1 < color2)
+  pixel1 = clutter_color_to_pixel (color1);
+  pixel2 = clutter_color_to_pixel (color2);
+
+  if (pixel1 < pixel2)
     return -1;
-  else if (color1 == color2)
+  else if (pixel1 == pixel2)
     return 0;
   else
     return 1;
 }
 
-static const GTypeValueTable _clutter_color_value_table = {
-  clutter_value_init_color,
-  clutter_value_free_color,
-  clutter_value_copy_color,
-  clutter_value_peek_color,
-  "p",
-  clutter_value_collect_color,
-  "p",
-  clutter_value_lcopy_color
-};
-
 GType
 clutter_param_color_get_type (void)
 {
