@@ -192,8 +192,6 @@ typedef enum
   CLUTTER_STAGE_STATE_ACTIVATED        = (1<<3)
 } ClutterStageState;
 
-typedef union _ClutterEvent ClutterEvent;
-
 typedef struct _ClutterAnyEvent         ClutterAnyEvent;
 typedef struct _ClutterButtonEvent      ClutterButtonEvent;
 typedef struct _ClutterKeyEvent         ClutterKeyEvent;
@@ -419,7 +417,6 @@ struct _ClutterStageStateEvent
 
 /**
  * ClutterEvent:
- * @type: event type
  *
  * Generic event wrapper.
  *
@@ -427,6 +424,7 @@ struct _ClutterStageStateEvent
  */
 union _ClutterEvent
 {
+  /*< private >*/
   ClutterEventType type;
 
   ClutterAnyEvent any;
