@@ -87,10 +87,7 @@ cogl_debug_flags_get_type (void)
   if (g_once_init_enter (&g_enum_type_id__volatile))
     {
       static const GFlagsValue values[] = {
-        { COGL_DEBUG_MISC, "COGL_DEBUG_MISC", "misc" },
-        { COGL_DEBUG_TEXTURE, "COGL_DEBUG_TEXTURE", "texture" },
-        { COGL_DEBUG_MATERIAL, "COGL_DEBUG_MATERIAL", "material" },
-        { COGL_DEBUG_SHADER, "COGL_DEBUG_SHADER", "shader" },
+        { COGL_DEBUG_SLICING, "COGL_DEBUG_SLICING", "slicing" },
         { COGL_DEBUG_OFFSCREEN, "COGL_DEBUG_OFFSCREEN", "offscreen" },
         { COGL_DEBUG_DRAW, "COGL_DEBUG_DRAW", "draw" },
         { COGL_DEBUG_PANGO, "COGL_DEBUG_PANGO", "pango" },
@@ -107,6 +104,7 @@ cogl_debug_flags_get_type (void)
         { COGL_DEBUG_ATLAS, "COGL_DEBUG_ATLAS", "atlas" },
         { COGL_DEBUG_DUMP_ATLAS_IMAGE, "COGL_DEBUG_DUMP_ATLAS_IMAGE", "dump-atlas-image" },
         { COGL_DEBUG_DISABLE_ATLAS, "COGL_DEBUG_DISABLE_ATLAS", "disable-atlas" },
+        { COGL_DEBUG_OPENGL, "COGL_DEBUG_OPENGL", "opengl" },
         { 0, NULL, NULL }
       };
       GType g_enum_type_id;
@@ -150,6 +148,29 @@ cogl_material_filter_get_type (void)
   return g_enum_type_id__volatile;
 }
 GType
+cogl_material_wrap_mode_get_type (void)
+{
+  static volatile gsize g_enum_type_id__volatile = 0;
+
+  if (g_once_init_enter (&g_enum_type_id__volatile))
+    {
+      static const GEnumValue values[] = {
+        { COGL_MATERIAL_WRAP_MODE_REPEAT, "COGL_MATERIAL_WRAP_MODE_REPEAT", "repeat" },
+        { COGL_MATERIAL_WRAP_MODE_CLAMP_TO_EDGE, "COGL_MATERIAL_WRAP_MODE_CLAMP_TO_EDGE", "clamp-to-edge" },
+        { COGL_MATERIAL_WRAP_MODE_AUTOMATIC, "COGL_MATERIAL_WRAP_MODE_AUTOMATIC", "automatic" },
+        { 0, NULL, NULL }
+      };
+      GType g_enum_type_id;
+
+      g_enum_type_id =
+        g_enum_register_static (g_intern_static_string ("CoglMaterialWrapMode"), values);
+
+      g_once_init_leave (&g_enum_type_id__volatile, g_enum_type_id);
+    }
+
+  return g_enum_type_id__volatile;
+}
+GType
 cogl_material_alpha_func_get_type (void)
 {
   static volatile gsize g_enum_type_id__volatile = 0;
@@ -287,6 +308,7 @@ cogl_feature_flags_get_type (void)
         { COGL_FEATURE_STENCIL_BUFFER, "COGL_FEATURE_STENCIL_BUFFER", "stencil-buffer" },
         { COGL_FEATURE_VBOS, "COGL_FEATURE_VBOS", "vbos" },
         { COGL_FEATURE_PBOS, "COGL_FEATURE_PBOS", "pbos" },
+        { COGL_FEATURE_UNSIGNED_INT_INDICES, "COGL_FEATURE_UNSIGNED_INT_INDICES", "unsigned-int-indices" },
         { 0, NULL, NULL }
       };
       GType g_enum_type_id;
@@ -458,6 +480,7 @@ cogl_indices_type_get_type (void)
       static const GEnumValue values[] = {
         { COGL_INDICES_TYPE_UNSIGNED_BYTE, "COGL_INDICES_TYPE_UNSIGNED_BYTE", "byte" },
         { COGL_INDICES_TYPE_UNSIGNED_SHORT, "COGL_INDICES_TYPE_UNSIGNED_SHORT", "short" },
+        { COGL_INDICES_TYPE_UNSIGNED_INT, "COGL_INDICES_TYPE_UNSIGNED_INT", "int" },
         { 0, NULL, NULL }
       };
       GType g_enum_type_id;
