@@ -8,6 +8,16 @@
 
 G_BEGIN_DECLS
 
+/* enumerations from "./cogl-buffer.h" */
+GType cogl_buffer_usage_hint_get_type (void) G_GNUC_CONST;
+#define COGL_TYPE_BUFFER_USAGE_HINT (cogl_buffer_usage_hint_get_type())
+
+GType cogl_buffer_update_hint_get_type (void) G_GNUC_CONST;
+#define COGL_TYPE_BUFFER_UPDATE_HINT (cogl_buffer_update_hint_get_type())
+
+GType cogl_buffer_access_get_type (void) G_GNUC_CONST;
+#define COGL_TYPE_BUFFER_ACCESS (cogl_buffer_access_get_type())
+
 /* enumerations from "./cogl-debug.h" */
 GType cogl_debug_flags_get_type (void) G_GNUC_CONST;
 #define COGL_TYPE_DEBUG_FLAGS (cogl_debug_flags_get_type())
@@ -16,6 +26,9 @@ GType cogl_debug_flags_get_type (void) G_GNUC_CONST;
 GType cogl_material_filter_get_type (void) G_GNUC_CONST;
 #define COGL_TYPE_MATERIAL_FILTER (cogl_material_filter_get_type())
 
+GType cogl_material_wrap_mode_get_type (void) G_GNUC_CONST;
+#define COGL_TYPE_MATERIAL_WRAP_MODE (cogl_material_wrap_mode_get_type())
+
 GType cogl_material_alpha_func_get_type (void) G_GNUC_CONST;
 #define COGL_TYPE_MATERIAL_ALPHA_FUNC (cogl_material_alpha_func_get_type())
 
