@@ -16,9 +16,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library. If not, see <http://www.gnu.org/licenses/>.
+ *
+ *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -130,14 +130,16 @@ _cogl_framebuffer_init (CoglFramebuffer *framebuffer,
   framebuffer->modelview_stack  = _cogl_matrix_stack_new ();
   framebuffer->projection_stack = _cogl_matrix_stack_new ();
 
+  framebuffer->dirty_bitmasks   = TRUE;
+
   /* Initialise the clip stack */
-  _cogl_clip_stack_state_init (&framebuffer->clip_state);
+  _cogl_clip_state_init (&framebuffer->clip_state);
 }
 
 void
 _cogl_framebuffer_free (CoglFramebuffer *framebuffer)
 {
-  _cogl_clip_stack_state_destroy (&framebuffer->clip_state);
+  _cogl_clip_state_destroy (&framebuffer->clip_state);
 
   _cogl_matrix_stack_destroy (framebuffer->modelview_stack);
   framebuffer->modelview_stack = NULL;
@@ -160,7 +162,7 @@ _cogl_framebuffer_get_height (CoglHandle handle)
   return framebuffer->height;
 }
 
-CoglClipStackState *
+CoglClipState *
 _cogl_framebuffer_get_clip_state (CoglHandle handle)
 {
   CoglFramebuffer *framebuffer = COGL_FRAMEBUFFER (handle);
@@ -248,6 +250,20 @@ _cogl_framebuffer_get_projection_stack (CoglHandle handle)
   return framebuffer->projection_stack;
 }
 
+static inline void
+_cogl_framebuffer_init_bits (CoglFramebuffer *framebuffer)
+{
+  if (G_LIKELY (!framebuffer->dirty_bitmasks))
+    return;
+
+  GE( glGetIntegerv (GL_RED_BITS,   &framebuffer->red_bits)   );
+  GE( glGetIntegerv (GL_GREEN_BITS, &framebuffer->green_bits) );
+  GE( glGetIntegerv (GL_BLUE_BITS,  &framebuffer->blue_bits)  );
+  GE( glGetIntegerv (GL_ALPHA_BITS, &framebuffer->alpha_bits) );
+
+  framebuffer->dirty_bitmasks = FALSE;
+}
+
 static gboolean
 try_creating_fbo (CoglOffscreen *offscreen,
                   TryFBOFlags flags,
@@ -273,8 +289,11 @@ try_creating_fbo (CoglOffscreen *offscreen,
       )
     return FALSE;
 
-  /* We are about to generate and bind a new fbo, so when next flushing the
-   * journal, we will need to rebind the current framebuffer... */
+  /* We are about to generate and bind a new fbo, so we pretend to change framebuffer
+   * state so that the old framebuffer will be rebound again before drawing.
+   * The framebuffer state can't be changed while their are active entries, so flush
+   * first. */
+  _cogl_journal_flush ();
   ctx->dirty_bound_framebuffer = 1;
 
   /* Generate framebuffer */
@@ -551,7 +570,9 @@ _cogl_set_framebuffer_real (CoglFramebuffer *framebuffer)
    * we flush */
   _cogl_matrix_stack_dirty (framebuffer->modelview_stack);
   _cogl_matrix_stack_dirty (framebuffer->projection_stack);
-  _cogl_clip_stack_state_dirty (&framebuffer->clip_state);
+  _cogl_clip_state_dirty (&framebuffer->clip_state);
+
+  _cogl_framebuffer_init_bits (framebuffer);
 }
 
 void
@@ -681,6 +702,12 @@ _cogl_framebuffer_flush_state (CoglHandle handle,
         gl_viewport_y = framebuffer->height -
           (framebuffer->viewport_y + framebuffer->viewport_height);
 
+      COGL_NOTE (OPENGL, "Calling glViewport(%d, %d, %d, %d)",
+                 framebuffer->viewport_x,
+                 gl_viewport_y,
+                 framebuffer->viewport_width,
+                 framebuffer->viewport_height);
+
       GE (glViewport (framebuffer->viewport_x,
                       gl_viewport_y,
                       framebuffer->viewport_width,
@@ -691,7 +718,7 @@ _cogl_framebuffer_flush_state (CoglHandle handle,
   /* XXX: Flushing clip state may trash the modelview and projection
    * matrices so we must do it before flushing the matrices...
    */
-  _cogl_flush_clip_state (&framebuffer->clip_state);
+  _cogl_clip_state_flush (&framebuffer->clip_state);
 
   if (!(flags & COGL_FRAMEBUFFER_FLUSH_SKIP_MODELVIEW))
     _cogl_matrix_stack_flush_to_gl (framebuffer->modelview_stack,
@@ -701,3 +728,42 @@ _cogl_framebuffer_flush_state (CoglHandle handle,
                                   COGL_MATRIX_PROJECTION);
 }
 
+int
+cogl_framebuffer_get_red_bits (CoglHandle framebuffer)
+{
+  CoglFramebuffer *fb = COGL_FRAMEBUFFER (framebuffer);
+
+  _cogl_framebuffer_init_bits (fb);
+
+  return fb->red_bits;
+}
+
+int
+cogl_framebuffer_get_green_bits (CoglHandle framebuffer)
+{
+  CoglFramebuffer *fb = COGL_FRAMEBUFFER (framebuffer);
+
+  _cogl_framebuffer_init_bits (fb);
+
+  return fb->green_bits;
+}
+
+int
+cogl_framebuffer_get_blue_bits (CoglHandle framebuffer)
+{
+  CoglFramebuffer *fb = COGL_FRAMEBUFFER (framebuffer);
+
+  _cogl_framebuffer_init_bits (fb);
+
+  return fb->blue_bits;
+}
+
+int
+cogl_framebuffer_get_alpha_bits (CoglHandle framebuffer)
+{
+  CoglFramebuffer *fb = COGL_FRAMEBUFFER (framebuffer);
+
+  _cogl_framebuffer_init_bits (fb);
+
+  return fb->alpha_bits;
+}
