@@ -3,7 +3,7 @@
  *
  * An object oriented GL/GLES Abstraction/Utility Layer
  *
- * Copyright (C) 2007,2008,2009 Intel Corporation.
+ * Copyright (C) 2007,2008,2009,2010 Intel Corporation.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -16,9 +16,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library. If not, see <http://www.gnu.org/licenses/>.
+ *
+ *
  */
 
 #ifndef __COGL_INTERNAL_H
@@ -26,13 +26,11 @@
 
 #include "cogl.h"
 #include "cogl-matrix-stack.h"
+#include "cogl-bitmask.h"
 
-typedef enum
-{
-  COGL_MATRIX_MODELVIEW,
-  COGL_MATRIX_PROJECTION,
-  COGL_MATRIX_TEXTURE
-} CoglMatrixMode;
+#ifdef COGL_HAS_XLIB_SUPPORT
+#include <X11/Xlib.h>
+#endif
 
 typedef enum
 {
@@ -55,11 +53,11 @@ typedef struct _CoglBoxedValue
   gboolean transpose;
 
   union {
-    gfloat float_value[4];
-    gint int_value[4];
-    gfloat matrix[16];
-    gfloat *float_array;
-    gint *int_array;
+    float float_value[4];
+    int int_value[4];
+    float matrix[16];
+    float *float_array;
+    int *int_array;
     gpointer array;
   } v;
 } CoglBoxedValue;
@@ -67,9 +65,10 @@ typedef struct _CoglBoxedValue
 
 #ifdef COGL_GL_DEBUG
 
-const gchar *cogl_gl_error_to_string (GLenum error_code);
+const char *
+cogl_gl_error_to_string (GLenum error_code);
 
-#define GE(x...)                        G_STMT_START {  \
+#define GE(x)                           G_STMT_START {  \
   GLenum __err;                                         \
   (x);                                                  \
   while ((__err = glGetError ()) != GL_NO_ERROR)        \
@@ -80,9 +79,21 @@ const gchar *cogl_gl_error_to_string (GLenum error_code);
                  cogl_gl_error_to_string (__err));      \
     }                                   } G_STMT_END
 
+#define GE_RET(ret, x)                  G_STMT_START {  \
+  GLenum __err;                                         \
+  ret = (x);                                            \
+  while ((__err = glGetError ()) != GL_NO_ERROR)        \
+    {                                                   \
+      g_warning ("%s: GL error (%d): %s\n",             \
+                 G_STRLOC,                              \
+                 __err,                                 \
+                 cogl_gl_error_to_string (__err));      \
+    }                                   } G_STMT_END
+
 #else /* !COGL_GL_DEBUG */
 
 #define GE(x) (x)
+#define GE_RET(ret, x)  (ret = (x))
 
 #endif /* COGL_GL_DEBUG */
 
@@ -92,11 +103,17 @@ const gchar *cogl_gl_error_to_string (GLenum error_code);
 #define COGL_ENABLE_COLOR_ARRAY       (1<<4)
 #define COGL_ENABLE_BACKFACE_CULLING  (1<<5)
 
-void    _cogl_features_init (void);
-gint    _cogl_get_format_bpp (CoglPixelFormat format);
+void
+_cogl_features_init (void);
 
-void    cogl_enable (gulong flags);
-gulong  cogl_get_enable (void);
+int
+_cogl_get_format_bpp (CoglPixelFormat format);
+
+void
+_cogl_enable (unsigned long flags);
+
+unsigned long
+_cogl_get_enable (void);
 
 typedef struct _CoglTextureUnit
 {
@@ -106,10 +123,55 @@ typedef struct _CoglTextureUnit
 
 CoglTextureUnit *
 _cogl_get_texture_unit (int index_);
+
 void
 _cogl_destroy_texture_units (void);
 
+unsigned int
+_cogl_get_max_texture_image_units (void);
+
 void
 _cogl_flush_face_winding (void);
 
+/* Disables the texcoord arrays that don't have a corresponding bit
+   set in the mask and sets ctx->texcoord_arrays_enabled to mask. Note
+   that it doesn't enable any extra texcoord arrays */
+void
+_cogl_disable_other_texcoord_arrays (const CoglBitmask *mask);
+
+#ifdef COGL_HAS_XLIB_SUPPORT
+
+/*
+ * CoglX11FilterReturn:
+ * @COGL_XLIB_FILTER_CONTINUE: The event was not handled, continues the
+ *                            processing
+ * @COGL_XLIB_FILTER_REMOVE: Remove the event, stops the processing
+ *
+ * Return values for the #CoglX11FilterFunc function.
+ */
+typedef enum _CoglXlibFilterReturn {
+  COGL_XLIB_FILTER_CONTINUE,
+  COGL_XLIB_FILTER_REMOVE
+} CoglXlibFilterReturn;
+
+/*
+ * cogl_xlib_handle_event:
+ * @xevent: pointer to XEvent structure
+ *
+ * This function processes a single X event; it can be used to hook
+ * into external X event retrieval (for example that done by Clutter
+ * or GDK).
+ *
+ * Return value: #CoglXlibFilterReturn. %COGL_XLIB_FILTER_REMOVE
+ * indicates that Cogl has internally handled the event and the
+ * caller should do no further processing. %COGL_XLIB_FILTER_CONTINUE
+ * indicates that Cogl is either not interested in the event,
+ * or has used the event to update internal state without taking
+ * any exclusive action.
+ */
+CoglXlibFilterReturn
+_cogl_xlib_handle_event (XEvent *xevent);
+
+#endif /* COGL_HAS_XLIB_SUPPORT */
+
 #endif /* __COGL_INTERNAL_H */
