@@ -263,7 +263,7 @@ _cogl_journal_flush_modelview_and_entries (CoglJournalEntry *batch_start,
                                   color_intensity : 0,
                                   0xff);
       _cogl_material_flush_gl_state (outline, NULL);
-      cogl_enable (COGL_ENABLE_VERTEX_ARRAY);
+      _cogl_enable (COGL_ENABLE_VERTEX_ARRAY);
       for (i = 0; i < batch_len; i++)
         GE( glDrawArrays (GL_LINE_LOOP, 4 * i + state->vertex_offset, 4) );
 
@@ -319,7 +319,7 @@ _cogl_journal_flush_material_and_entries (CoglJournalEntry *batch_start,
                                  &batch_start->flush_options);
 
   /* FIXME: This api is a bit yukky, ideally it will be removed if we
-   * re-work the cogl_enable mechanism */
+   * re-work the _cogl_enable mechanism */
   enable_flags |= _cogl_material_get_cogl_enable_flags (batch_start->material);
 
   if (ctx->enable_backface_culling)
@@ -327,7 +327,7 @@ _cogl_journal_flush_material_and_entries (CoglJournalEntry *batch_start,
 
   enable_flags |= COGL_ENABLE_VERTEX_ARRAY;
   enable_flags |= COGL_ENABLE_COLOR_ARRAY;
-  cogl_enable (enable_flags);
+  _cogl_enable (enable_flags);
   _cogl_flush_face_winding ();
 
   /* If we haven't transformed the quads in software then we need to also break
@@ -372,7 +372,6 @@ _cogl_journal_flush_texcoord_vbo_offsets_and_entries (
                                           void             *data)
 {
   CoglJournalFlushState *state = data;
-  int                    prev_n_texcoord_arrays_enabled;
   int                    i;
 
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
@@ -395,14 +394,10 @@ _cogl_journal_flush_texcoord_vbo_offsets_and_entries (
                                       (POS_STRIDE + COLOR_STRIDE) * 4 +
                                       TEX_STRIDE * 4 * i)));
     }
-  prev_n_texcoord_arrays_enabled =
-    ctx->n_texcoord_arrays_enabled;
-  ctx->n_texcoord_arrays_enabled = batch_start->n_layers;
-  for (; i < prev_n_texcoord_arrays_enabled; i++)
-    {
-      GE (glClientActiveTexture (GL_TEXTURE0 + i));
-      GE (glDisableClientState (GL_TEXTURE_COORD_ARRAY));
-    }
+
+  _cogl_bitmask_clear_all (&ctx->temp_bitmask);
+  _cogl_bitmask_set_range (&ctx->temp_bitmask, batch_start->n_layers, TRUE);
+  _cogl_disable_other_texcoord_arrays (&ctx->temp_bitmask);
 
   batch_and_call (batch_start,
                   batch_len,
@@ -542,7 +537,7 @@ upload_vertices_to_vbo (GArray *vertices, CoglJournalFlushState *state)
 
   /* As we flush the journal entries in batches we walk forward through the
    * above VBO starting at offset 0... */
-  state->vbo_offset = 0;
+  state->vbo_offset = NULL;
 
   return journal_vbo;
 }
@@ -663,6 +658,8 @@ _cogl_journal_log_quad (const float  *position,
                         int           n_layers,
                         guint32       fallback_layers,
                         GLuint        layer0_override_texture,
+                        const CoglMaterialWrapModeOverrides *
+                                      wrap_mode_overrides,
                         const float  *tex_coords,
                         unsigned int  tex_coords_len)
 {
@@ -803,6 +800,11 @@ _cogl_journal_log_quad (const float  *position,
     COGL_MATERIAL_FLUSH_SKIP_GL_COLOR;
   entry->flush_options.fallback_layers = fallback_layers;
   entry->flush_options.disable_layers = disable_layers;
+  if (wrap_mode_overrides)
+    {
+      entry->flush_options.flags |= COGL_MATERIAL_FLUSH_WRAP_MODE_OVERRIDES;
+      entry->flush_options.wrap_mode_overrides = *wrap_mode_overrides;
+    }
   if (layer0_override_texture)
     {
       entry->flush_options.flags |= COGL_MATERIAL_FLUSH_LAYER0_OVERRIDE;
