@@ -61,47 +61,6 @@ cogl_util_next_p2 (int a)
 
 /* gtypes */
 
-CoglHandle
-cogl_handle_ref (CoglHandle handle)
-{
-  CoglHandleObject *obj = (CoglHandleObject *)handle;
-
-  g_return_val_if_fail (handle != COGL_INVALID_HANDLE, COGL_INVALID_HANDLE);
-
-  obj->ref_count++;
-  return handle;
-}
-
-void
-cogl_handle_unref (CoglHandle handle)
-{
-  CoglHandleObject *obj = (CoglHandleObject *)handle;
-
-  g_return_if_fail (handle != COGL_INVALID_HANDLE);
-
-  if (--obj->ref_count < 1)
-    {
-      void (*free_func)(void *obj);
-
-      COGL_HANDLE_DEBUG_FREE (obj);
-      free_func = obj->klass->virt_free;
-      free_func (obj);
-    }
-}
-
-GType
-cogl_handle_get_type (void)
-{
-  static GType our_type = 0;
-
-  if (G_UNLIKELY (our_type == 0))
-    our_type = g_boxed_type_register_static (g_intern_static_string ("CoglHandle"),
-                                            (GBoxedCopyFunc) cogl_handle_ref,
-                                            (GBoxedFreeFunc) cogl_handle_unref);
-
-  return our_type;
-}
-
 /*
  * CoglFixed
  */
